from typing import TypeAlias, Literal, TYPE_CHECKING
if TYPE_CHECKING:
    from pyspark.sql import DataFrame




    ri_foundry_main_dataset_b0d1161d_5ae5_4e76_a6e2_9c7524eff80f:TypeAlias = 'DataFrame[Literal["id", "first_name", "last_name", "email", "gender", "ip_address", "comment", "image", "dt"]]'





    test2_95:TypeAlias = 'DataFrame[Literal["id", "first_name", "last_name", "email", "gender", "ip_address", "comment", "image", "dt"]]'
